<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @wordpress-plugin
 * Plugin Name:       Campation PostOffice
 * Plugin URI:        https://campation.com/
 * Description:       The ALL-IN-ONE solution for high-end email - 10 x faster than SMTP!
 * Version:           1.5.5
 * Author:            Campation LLC
 * License:           GPLv2 or later
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       campation-postoffice
 * Domain Path:       /languages/
 * Requires PHP:      7.2
 * Requires at least: 5.4
 * Tested up to:      6.1
 *
 * @link              https://campation.com/
 * @since             1.0.0
 * @author            Campation LLC <info@campation.com>
 * @copyright         Copyright 2021-22 Campation LLC
 *
 * LICENSE
 * Campation PostOffice (TM) is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
              */
 
// exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {	exit(); }


// constants
define("CPTNPONAME","Campation PostOffice&trade;");
define("CPTNPOVERSION","1.5.5");



// add action links to plugin page
add_filter( 'plugin_action_links_campation-postoffice/campation-postoffice.php', 'cptn_po_filter_action_links', 10, 1 );
function cptn_po_filter_action_links( $links ) {
   $links['setup']    = '<a href="'.admin_url('admin.php?page=campation-postoffice-assistant').'">'. __('Setup wizard', 'campation-postoffice').'</a>';
   $links['settings'] = '<a href="'.admin_url('admin.php?page=campation-postoffice-settings').'">'. __('Settings', 'campation-postoffice').'</a>';
   $links['support']  = '<a href="'.admin_url('admin.php?page=campation-postoffice-faq-support').'">'. __('Support', 'campation-postoffice').'</a>';
   return $links;
   }

// add js to deactivation link
global $pagenow;
if ($pagenow == 'plugins.php')
   add_action('admin_footer', 'cptn_po_deactivation_js');
function cptn_po_deactivation_js()
   {
?>
   <script type="text/javascript">
      jQuery(document).ready(function($) {
         $(".deactivate a[href*='campation-postoffice.php']").click(function(e) {
		    alert("<?php if(get_option("cptn_po_opt_delete_data_uninstall")==0) 
			                echo __("If you intend to disable AND delete the plugin: the plugin's data currently will be PRESERVED. Change: Campation PostOffice -> Settings -> General -> Erase all data ...", 'campation-postoffice'); 
			             else
			                echo __("If you intend to disable AND delete the plugin: the plugin's data will be ERASED. Change: Campation PostOffice -> Settings -> General -> Erase all data ...", 'campation-postoffice'); 
			       ?>");
		    });
		 });
   </script>
<?php
   }   
 
   
   
   
   
// do we speak SSL?
if ( (isset($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME'] == 'https') ||
     (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ||
     (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == '443') ) 
   define("CPTNPOPATH",str_replace("http://","https://",plugin_dir_url( __FILE__ )));
else 
   define("CPTNPOPATH",str_replace("https://","http://",plugin_dir_url( __FILE__ )));

   
// own IP of server unknown
if(php_sapi_name() !== 'cli' AND get_option("cptn_po_server_ip")=="")
   {
   $ip = "";
   if(isset($_SERVER['SERVER_ADDR']) AND !empty($_SERVER['SERVER_ADDR'])) $ip = $_SERVER['SERVER_ADDR'];
   elseif(null !== getenv('SERVER_ADDR') AND !empty(getenv('SERVER_ADDR'))) $ip = getenv('SERVER_ADDR');
   else 
      {
	  $response = @wp_remote_get("https://api.ipify.org");
	  if(is_array($response) && !is_wp_error($response))
	     {
    	 $headers = $response['headers'];
    	 $ip      = $response['body'];
		 }	  
      else
	     {
	  	 $response = @wp_remote_get("https://ipinfo.io/ip");
	  	 if(is_array($response) && !is_wp_error($response))
	        {
    	 	$headers = $response['headers'];
    	 	$ip      = $response['body'];
		 	}	  
		 }
	  }	  
   if($ip)
      {
   	  update_option("cptn_po_server_ip", $ip);
	  }
   define("CPTNPOIP",$ip);
   }
else
   {
   $ip = get_option("cptn_po_server_ip");
   define("CPTNPOIP",$ip);
   }
stream_context_get_default(array('socket'=>array('bindto'=>CPTNPOIP.':0'))); 
      
   
define("CPTNPODIR",plugin_dir_path( __FILE__ ));              // with closing slash!
define("CPTN_PO_MAXRETRIES",5);               				  // maximum retries sending mail
define("CPTN_PO_RETRY_MIN",[15,45,120,360,720]);			  // minute steps for retries
define("CPTNPOREADONLINEURL",home_url("/cptnporeadonline/","https"));  // read online - add message id
define("CPTNPOAFFILIATEURL","https://r.freemius.com/9582/");  // freemius affiliate link - add affiliate id
define("CPTNPOAFFILIATEINFO","/wp-admin/admin.php?page=campation-postoffice-affiliation");
define("CPTNPOAPIURL",'https://api.campation.com/');
if(!defined("CPTNVERSIONPHP"))           define("CPTNVERSIONPHP","7.2");
if(!defined("CPTNVERSIONMYSQL")) 		 define("CPTNVERSIONMYSQL","5.7");
if(!defined("CPTNVERSIONMARIADB")) 		 define("CPTNVERSIONMARIADB","10.2");
if(!defined("CPTNDATETIMEFORMAT")) 		 define("CPTNDATETIMEFORMAT",esc_html(get_option('date_format'))." ".esc_html(get_option('time_format')));
if(!defined("CPTNDATEFORMAT")) 			 define("CPTNDATEFORMAT",esc_html(get_option('date_format')));
if(!defined("CPTNTIMEFORMAT")) 			 define("CPTNTIMEFORMAT",esc_html(get_option('time_format')));
if(!defined("PARDOT_API_CACHE_TIMEOUT")) define("PARDOT_API_CACHE_TIMEOUT", 30);   // tinymce cache
if(!defined("CPTNWOOCOMMERCE")) 		 define("CPTNWOOCOMMERCE",class_exists('WooCommerce'));
if(!defined('CPTNSUPPORTURL')) 			 define("CPTNSUPPORTURL","https://support.campation.com/");





// HTML email header for Admin messages
$domain = str_replace("https://","",str_replace("http://","",str_replace("www.","",get_home_url())));
$senderemail = esc_html(get_option("cptn_po_opt_envelope_account_email"));
if(empty($senderemail))
   $senderemail = esc_html(get_option("admin_email"));
$headers  = "From: Campation PostOffice (".mb_encode_mimeheader(trim($domain)).") <".$senderemail.">\r\n";
$headers .= "MIME-Version: 1.0\r\n";
$headers .= "Content-Transfer-Encoding: 8bit\r\n";
$headers .= "Content-Type: text/html; charset=utf-8\r\n";
$headers .= "Return-Path: ".$senderemail."\r\n";
define("CPTNPOHEADERS",$headers);              

// load includes
include_once("common/includes/cptn-general-functions.php");
include_once("includes/cptn-po-functions.php");
include_once("backend/assets/includes/cptn-po-backend-functions.php");
include_once("common/includes/cptn-database.php");
include_once("common/ajax/common-ajax.php");



// START FREEMIUS
if ( function_exists( 'cptn_po_fs' ) ) 
   cptn_po_fs()->set_basename( false, __FILE__ );
else 
   include_once("backend/assets/includes/cptn-po-freemius.php");

// freemius uninstall hook
cptn_po_fs()->add_action('after_uninstall', 'cptn_po_uninstall_cleanup');
  
include_once("cron/cptn-po-cron.php");
include_once("cron/cptn-po-cron-developer-report.php");




   // activation / deactivation hooks
   register_activation_hook(__FILE__, 'cptn_po_activation_requirements');
   if(!function_exists("cptn_po_activation_requirements"))
      {
   	  function cptn_po_activation_requirements ()
         {
	  	 // check requirements PHP and MySQL
   	  	 global $wpdb;
   	  	 $query = 'SHOW VARIABLES LIKE "version";';
   	  	 $versionresult = $wpdb->get_row($query,ARRAY_A);
   	  	 $mysqlVersion = $versionresult['Value']; 

   	     $query = 'SHOW VARIABLES LIKE "version_comment";';
   	  	 $vendorresult = $wpdb->get_row($query,ARRAY_A);
   	  	 $mysqlVendor = $vendorresult['Value']; 
  
   	     if((strpos(strtolower($mysqlVendor),"percona")!==false OR strpos(strtolower($mysqlVendor),"mysql")!==false) AND version_compare($mysqlVersion, CPTNVERSIONMYSQL) < 0)  // MySQL
            {
	  	 	deactivate_plugins(CPTNPODIR.'campation-postoffice.php');
	  	 	wp_die(esc_html__('Campation PostOffice&trade; requires at least',"campation-postoffice").' <strong><u>MySQL '.CPTNVERSIONMYSQL.'</u></strong>. '.esc_html__('You have installed',"campation-postoffice").' '.esc_html($mysqlVersion).'. '.esc_html__('Plugin cannot be activated',"campation-postoffice").'. <a href="javascript:history.back()">'.esc_html__('Go Back',"campation-postoffice").'</a>');
	     	}
         elseif(strpos(strtolower($mysqlVendor),"mariadb")!==false AND version_compare($mysqlVersion, CPTNVERSIONMARIADB) < 0)  // MariaDB
            {
	  	 	deactivate_plugins(CPTNPODIR.'campation-postoffice.php');
	  	 	wp_die(esc_html__('Campation PostOffice&trade; requires at least',"campation-postoffice").' <strong><u>MariaDB '.CPTNVERSIONMARIADB.'</u></strong>. '.esc_html__('You have installed',"campation-postoffice").' '.esc_html($mysqlVersion).'. '.esc_html__('Plugin cannot be activated',"campation-postoffice").'. <a href="javascript:history.back()">'.esc_html__('Go Back',"campation-postoffice").'</a>');
	     	}
         if(version_compare(phpversion(), CPTNVERSIONPHP) < 0) 
            {
	  	 	deactivate_plugins(CPTNPODIR.'campation-postoffice.php');
	  	 	wp_die(esc_html__('Campation PostOffice&trade; requires at least',"campation-postoffice").' <strong><u>PHP '.CPTNVERSIONPHP.'</u></strong>. '.esc_html__('You have installed',"campation-postoffice").' '.phpversion().'. '.esc_html__('Plugin cannot be activated',"campation-postoffice").'. <a href="javascript:history.back()">'.esc_html__('Go Back',"campation-postoffice").'</a>');
	  	 	}

         // is there another wp_mail() plugin active?
		 $blockingplugin = "";
         try {
             $mirror            = new ReflectionFunction( 'wp_mail' );
             $is_default_wpmail = substr( $mirror->getFileName(), -strlen( 'pluggable.php' ) ) === 'pluggable.php';
             $is_cptn_po_mail   = substr( $mirror->getFileName(), -strlen( 'cptn-po-wp-mail.php' ) ) === 'cptn-po-wp-mail.php';
			 $blockingplugin    = str_replace(".php","",substr($mirror->getFileName(), strrpos($mirror->getFileName(),"/")+1,100));
         } catch ( Exception $e ) {
             $is_default_wpmail = true;
             $is_cptn_po_mail   = false;
            }
         if(!$is_default_wpmail AND !$is_cptn_po_mail)
            {
	  	 	deactivate_plugins(CPTNPODIR.'campation-postoffice.php');
	  	 	wp_die(esc_html__('Another plugin is blocking the wp_mail() function: ',"campation-postoffice").'<strong>'.strtoupper($blockingplugin).'</strong> - '.esc_html__('Plugin cannot be activated',"campation-postoffice").'. <a href="javascript:history.back()">'.esc_html__('Go Back',"campation-postoffice").'</a>');
			}
			
   	     include_once("includes/cptn-po-sender-check.php");
		 cptn_db_install();
		 cptn_po_db_install();
		 include_once("includes/cptn-po-upgraders.php");
		 cptn_po_cron_activate();
		 flush_rewrite_rules();
		 cptn_po_sender_check(1);
		 error_log("Plugin 'Campation PostOffice' ".CPTNPOVERSION." was activated");
   		 update_option("cptn_po_opt_version",CPTNPOVERSION);
		 if(get_option("cptn_po_opt_assistant_status") != "OK")
			{
			update_option("cptn_po_opt_assistant_step","1");
			header("Location: /wp-admin/admin.php?page=campation-postoffice-assistant");
			}
	     cptn_po_cron_developer_report("ACTIVATE");
 		 }
      }

   register_deactivation_hook( __FILE__, 'cptn_po_cron_deactivate' );



   // plugin updated? new version number higher than saved	
   add_action('plugins_loaded', 'cptn_po_plugin_update',3);  
   function cptn_po_plugin_update() 
      {
   	  global $wpdb;
   	  if (version_compare(CPTNPOVERSION, get_option("cptn_po_opt_version"))==1 AND get_option("cptn_po_opt_version") AND !get_transient("cptn_po_updating"))
	     {
		 set_transient("cptn_po_updating",1); 
	  	 cptn_db_install();
	  	 cptn_po_db_install();
	  	 include_once("includes/cptn-po-upgraders.php");
	  	 cptn_po_cron_deactivate();
	  	 cptn_po_cron_activate();
	  	 flush_rewrite_rules();
	  	 update_option("cptn_po_opt_assistant_status", "UPDATE");
	  	 update_option("cptn_po_opt_assistant_step","2");
	  	 error_log("Plugin 'Campation PostOffice' was updated from ".get_option("cptn_po_opt_version")." to ".CPTNPOVERSION);
   	  	 update_option("cptn_po_opt_version",CPTNPOVERSION);
	     cptn_po_cron_developer_report("UPDATE");
		 delete_transient("cptn_po_updating"); 
	  	 }
   	  }


  



   // initialize plugin after loading   
   add_action('plugins_loaded', 'cptn_po_text_domain',2);  
   function cptn_po_text_domain() 
      {
   	  global $wpdb;
   
   	  // load translations and cut local parts from locale (de_DE_formal, es_CO
   	  $locale = apply_filters("plugin_locale", get_locale(), "campation-postoffice" );
      if($locale=="de_DE_formal") 			     $locale = "de_DE_formal";
   	  elseif(substr($locale,0,2)=="de")      	 $locale = "de_DE";
   	  elseif(substr($locale,0,2)=="es")      	 $locale = "es_ES";
   	  add_filter('locale',function($locale) {return $locale;},10);  
   	  load_textdomain("campation-postoffice", CPTNPODIR."languages/campation-postoffice-".esc_html($locale).".mo");
   	  }




   // include backend
   if(is_admin())
      {
      // load scripts and css
   	  if(cptn_po_is_plugin_page("*"))
         {
      	 include_once("backend/assets/includes/cptn-po-load-scripts.php");
	  	 include_once("common/css/cptn-general-css.php");
	  	 }

   	  include_once("backend/assets/ajax/general-ajax.php");

      if(cptn_po_is_plugin_page("*"))
   	     include_once("backend/index.php");

      // admin menu	  
   	  add_action('admin_menu', 'cptn_po_setup_menu');
   	  function cptn_po_setup_menu()
	     {
		 $versiontype = "";
		 if(cptn_po_fs()->is_trial())  
		    $versiontype = '<sup><span style="color:#999;font-size:0.8em;margin-left:3px;">Trial</span></sup>';
		 elseif(cptn_po_fs()->is_plan('business'))  
		    $versiontype = '<sup><span style="color:#999;font-size:0.8em;margin-left:3px;">Business</span></sup>';
		 elseif(cptn_po_fs()->is_plan('premium'))  
		    $versiontype = '<sup><span style="color:#999;font-size:0.8em;margin-left:3px;">Premium</span></sup>';
		 elseif(cptn_po_fs()->is_plan('starter'))  
		    $versiontype = '<sup><span style="color:#999;font-size:0.8em;margin-left:3px;">Starter</span></sup>';
		 else  
		    $versiontype = '<sup><span style="color:#999;font-size:0.8em;margin-left:3px;">free</span></sup>';

         add_menu_page( 'Campation PostOffice&trade;', 'Campation PostOffice&trade;'.$versiontype, 'manage_options', 'campation-postoffice', 'cptn_po_admin_cockpit', CPTNPOPATH.'img/wp-icon.png',7);
   	  	 add_submenu_page('campation-postoffice', esc_html__('Cockpit',"campation-postoffice"), esc_html__('Cockpit',"campation-postoffice"), 'manage_options','campation-postoffice', 'cptn_po_admin_cockpit');
   	  	 add_submenu_page('campation-postoffice', esc_html__('Outbox',"campation-postoffice"), esc_html__('Outbox',"campation-postoffice"), 'manage_options','campation-postoffice-outbox', 'cptn_po_admin_outbox');
   	  	 add_submenu_page('campation-postoffice', esc_html__('Recipients',"campation-postoffice"), esc_html__('Recipients',"campation-postoffice"), 'manage_options','campation-postoffice-recipient', 'cptn_po_admin_recipient');
   	  	 add_submenu_page('campation-postoffice', esc_html__('Mail servers',"campation-postoffice"), esc_html__('Mail servers',"campation-postoffice"), 'manage_options','campation-postoffice-mailserver', 'cptn_po_admin_mailserver');
   	  	 add_submenu_page('campation-postoffice', esc_html__('Domains',"campation-postoffice"), esc_html__('Domains',"campation-postoffice"), 'manage_options','campation-postoffice-domain', 'cptn_po_admin_domain');
   	  	 add_submenu_page('campation-postoffice', esc_html__('Settings',"campation-postoffice"), esc_html__('Settings',"campation-postoffice"), 'manage_options', 'campation-postoffice-settings', 'cptn_po_admin_configuration');
  	  	 add_submenu_page('campation-postoffice', esc_html__('Setup wizard',"campation-postoffice"), esc_html__('Setup wizard',"campation-postoffice"), 'manage_options', 'campation-postoffice-assistant', 'cptn_po_admin_assistant');
   	  	 add_submenu_page('campation-postoffice', esc_html__('FAQ / Support',"campation-postoffice"), esc_html__('FAQ / Support',"campation-postoffice"), 'manage_options', 'campation-postoffice-faq-support', 'cptn_admin_support');
   	     add_submenu_page('campation-postoffice', esc_html__('Why PRO?',"campation-postoffice"), esc_html__('Why PRO?',"campation-postoffice"), 'manage_options', 'campation-postoffice-whypro', 'cptn_po_admin_whypro');
   	  	 }

      // remove replacement of UTF-8 icons by emojis (since WP 6.0)
	  if ( !function_exists( 'cptn_po_disable_emojis_admin' ) ) 
   	     {
   		 add_action( 'admin_init', 'cptn_po_disable_emojis_admin' );
   		 function cptn_po_disable_emojis_admin() {
      	    remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
      		}
         }

      }  // END is_admin()

   
   // load rest of includes
   include_once("common/includes/cptn-kses-functions.php");
   include_once("includes/cptn-po-wp-mail.php");
   include_once("includes/cptn-po-message-functions.php");
   include_once("includes/cptn-po-server-test-functions.php");
   include_once("includes/cptn-po-virtual-urls.php");
   include_once("includes/cptn-po-sender-check.php");
   include_once("includes/cptn-po-uninstall.php");
   include_once("backend/assets/includes/cptn-po-database.php");
   include_once("backend/assets/ajax/cockpit-ajax.php");
   include_once("backend/assets/ajax/recipients-ajax.php");
   include_once("backend/assets/ajax/outbox-ajax.php");
   include_once("backend/assets/ajax/domains-ajax.php");
   include_once("backend/assets/ajax/mailserver-ajax.php");
   include_once("backend/assets/ajax/general-ajax.php");
   include_once("backend/assets/ajax/configuration-ajax.php");
   include_once("backend/assets/ajax/assistant-ajax.php");
   include_once("backend/assets/includes/cptn-po-admin-bar.php");
   include_once("backend/assets/includes/cptn-po-freemius-adapt.php");
   include_once("common/ajax/support-ajax.php");

   if(!cptn_po_fs()->can_use_premium_code())
      define("CPTNPO_FS",'free');
   else
      {
      if(cptn_po_fs()->is_plan('business'))
         define("CPTNPO_FS",'business');
      elseif(cptn_po_fs()->is_plan('premium'))
         define("CPTNPO_FS",'premium');
   	  elseif(cptn_po_fs()->is_plan('starter'))
         define("CPTNPO_FS",'starter');
      }   
	  







?>