<?php
// sanitize html and arrays
if(!function_exists("cptn_recursive_sanitize_array"))
   {
   function cptn_recursive_sanitize_array($array) 
      {
      global $allowedposttags;
	  $allowedposttags['center'] = array();
	  $allowedposttags['style'] = array();

	  $allowed_atts = array('align'=>array(), 'class'=>array(), 'id'=>array(), 'dir'=>array(), 'lang'=>array(), 'style'=>array(), 'xml:lang'=>array(), 'src'=>array(), 'alt'=>array(), 'href'=>array(), 'target'=>array(), 'download'=>array());
	  $allowedposttags['strong'] = $allowed_atts;
	  $allowedposttags['small'] = $allowed_atts;
	  $allowedposttags['span'] = $allowed_atts;
	  $allowedposttags['abbr'] = $allowed_atts;
	  $allowedposttags['code'] = $allowed_atts;
	  $allowedposttags['div'] = $allowed_atts;
	  $allowedposttags['img'] = $allowed_atts;
	  $allowedposttags['h1'] = $allowed_atts;
	  $allowedposttags['h2'] = $allowed_atts;
	  $allowedposttags['h3'] = $allowed_atts;
	  $allowedposttags['h4'] = $allowed_atts;
	  $allowedposttags['h5'] = $allowed_atts;
	  $allowedposttags['ol'] = $allowed_atts;
	  $allowedposttags['ul'] = $allowed_atts;
	  $allowedposttags['li'] = $allowed_atts;
	  $allowedposttags['em'] = $allowed_atts;
	  $allowedposttags['p'] = $allowed_atts;
	  $allowedposttags['a'] = $allowed_atts;
	  
   	  foreach ( $array as $key => &$value ) 
	     {
		 if(strpos($key,"dkim_")!==false AND strpos($key,"_key")!==false)                    // DKIM keys, have LF to be preserved
		    $value = sanitize_textarea_field($value);
         elseif(is_array($value))                                                            // Array
            $value = cptn_recursive_sanitize_array($value);
         elseif($value AND (strpos(strtolower($value),"from:")!==false                					 // Email header
		    OR strpos(strtolower($value),"to:")!==false
			OR strpos(strtolower($value),"cc:")!==false
			OR strpos(strtolower($value),"bcc:")!==false
			OR strpos(strtolower($value),"reply-to:")!==false
			))
            $value = preg_replace('/[^\p{L}\p{N}\.\-\_\@\:\=\?\<\>\/\(\) ]/u', '', $value);
         elseif(filter_var($value, FILTER_VALIDATE_EMAIL))                                   // Email address
            $value = sanitize_email($value);
         elseif($value AND $value != strip_tags($value))   			                                 // HTML
            $value = stripslashes(wp_kses_post($value,$allowedposttags));
         elseif($value AND $value == strip_tags($value) AND strpos($value,PHP_EOL)!==false)   			 // Text with CR LF
            $value = sanitize_textarea_field($value);
         else	   	 													 					 // all the rest
            $value = sanitize_text_field($value);
         }
   	  return $array;
      }
   }


   
if(!function_exists("cptn_kses_filter_allowed_html"))
   {
   function cptn_kses_filter_allowed_html ($allowed)
      {
	  $allowed['span']['data-*']	 = true;
	  $allowed['i']['data-*']	 	 = true;
      $allowed['a']['data-*']        = true;
      $allowed['tr']['data-*']       = true;
      $allowed['td']['data-*']       = true;
      $allowed['table']['data-*'] 	 = true; 
      $allowed['select']['data-*'] 	 = true; 
      $allowed['span']['data-*'] 	 = true; 
      return $allowed;
      }   
   add_filter("wp_kses_allowed_html", "cptn_kses_filter_allowed_html", 10, 1);
   }   


if(!function_exists("cptn_kses_allowed_tags"))
   {
   function cptn_kses_allowed_tags()
      {
      global $allowedposttags;
	  $allowed_atts = array('align'       => true,
	  				        'valign'	  => true, 
      				        'frameborder' => true,
    						'height'      => true,
    						'width'       => true,
    						'sandbox'     => true,
    						'seamless'    => true,
    						'scrolling'   => true,
    						'srcdoc'      => true,
    						'src'         => true,
    						'class'       => true,
    						'id'          => true,
    						'style'       => true,
    						'border'      => true,
							'dir'		  => true, 
							'lang'		  => true, 
							'xml:lang'	  => true, 
							'alt'		  => true, 
							'href'		  => true, 
							'target'	  => true, 
							'download'	  => true,
							'type'		  => true,
							'min'		  => true,
							'max'		  => true,
							'step'		  => true,
							'name'		  => true,
							'content'	  => true,
							'value'		  => true,
							'height'	  => true,
							'width'		  => true,
							'colspan'	  => true,
							'cellspacing' => true,
							'title'		  => true,
							'loading'	  => true,
							'http-equiv'  => true,
							'data-*'	  => true
							);

	  $allowedposttags['html'] 	 	 = $allowed_atts;
	  $allowedposttags['meta'] 	 	 = $allowed_atts;
	  $allowedposttags['head'] 		 = $allowed_atts;
	  $allowedposttags['style'] 	 = $allowed_atts;
	  $allowedposttags['body'] 		 = $allowed_atts;
	  $allowedposttags['iframe'] 	 = $allowed_atts;
	  $allowedposttags['strong'] 	 = $allowed_atts;
	  $allowedposttags['small'] 	 = $allowed_atts;
	  $allowedposttags['span'] 		 = $allowed_atts;
	  $allowedposttags['abbr'] 		 = $allowed_atts;
	  $allowedposttags['sup'] 		 = $allowed_atts;
	  $allowedposttags['code'] 		 = $allowed_atts;
	  $allowedposttags['div'] 		 = $allowed_atts;
	  $allowedposttags['img'] 		 = $allowed_atts;
	  $allowedposttags['h1'] 		 = $allowed_atts;
	  $allowedposttags['h2'] 		 = $allowed_atts;
	  $allowedposttags['h3'] 		 = $allowed_atts;
	  $allowedposttags['h4'] 		 = $allowed_atts;
	  $allowedposttags['h5'] 		 = $allowed_atts;
	  $allowedposttags['ol'] 		 = $allowed_atts;
	  $allowedposttags['ul'] 		 = $allowed_atts;
	  $allowedposttags['li'] 		 = $allowed_atts;
	  $allowedposttags['em'] 		 = $allowed_atts;
	  $allowedposttags['p'] 		 = $allowed_atts;
	  $allowedposttags['a'] 		 = $allowed_atts;
	  $allowedposttags['i'] 		 = $allowed_atts;
	  $allowedposttags['option'] 	 = $allowed_atts;
	  $allowedposttags['form'] 	 	 = $allowed_atts;
	  $allowedposttags['input'] 	 = $allowed_atts;
	  $allowedposttags['select'] 	 = $allowed_atts;
	  $allowedposttags['textarea'] 	 = $allowed_atts;
	  $allowedposttags['table'] 	 = $allowed_atts;
	  $allowedposttags['tr'] 	 	 = $allowed_atts;
	  $allowedposttags['td'] 	 	 = $allowed_atts;
	  $allowedposttags['center'] 	 = $allowed_atts;


	  $allowedposttags['!doctype html'] 	 = $allowed_atts;


      }   
   add_action("init", "cptn_kses_allowed_tags");
   }   


add_filter( 'safe_style_css', function( $styles ) {
   $styles[] = 'text-overflow';
   $styles[] = 'white-space';
   $styles[] = 'overflow-wrap';
   $styles[] = 'table-layout';
   $styles[] = 'max-width';
   $styles[] = 'display';
   $styles[] = 'opacity';
   $styles[] = 'mso-hide';
   return $styles;
   });   
   
   
   
if(!function_exists("cptn_json_output"))
   {
   function cptn_json_output ($output)
      {
   	  $output = json_encode(cptn_recursive_sanitize_array($output));
   	  $output = str_replace(':null',':""',$output);
   	  $output = str_replace(':false',':""',$output);
   	  return($output);
   	  }
   }   

?>