<?php

// Add custom cron interval to every page
add_filter( 'cron_schedules', 'cptn_po_custom_cron_intervals' );
function cptn_po_custom_cron_intervals( $schedules )
{
    if ( !isset( $schedules['every-1-minutes'] ) ) {
        $schedules['every-1-minutes'] = array(
            'interval' => 60,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every minute', "campation-postoffice" ) ),
        );
    }
    if ( !isset( $schedules['every-5-minutes'] ) ) {
        $schedules['every-5-minutes'] = array(
            'interval' => 60 * 5,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every 5 minutes', "campation-postoffice" ) ),
        );
    }
    if ( !isset( $schedules['every-15-minutes'] ) ) {
        $schedules['every-15-minutes'] = array(
            'interval' => 60 * 15,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every 15 minutes', "campation-postoffice" ) ),
        );
    }
    if ( !isset( $schedules['every-30-minutes'] ) ) {
        $schedules['every-30-minutes'] = array(
            'interval' => 60 * 30,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every 30 minutes', "campation-postoffice" ) ),
        );
    }
    if ( !isset( $schedules['hourly'] ) ) {
        $schedules['hourly'] = array(
            'interval' => 60 * 60,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every hour', "campation-postoffice" ) ),
        );
    }
    if ( !isset( $schedules['daily'] ) ) {
        $schedules['daily'] = array(
            'interval' => 60 * 60 * 24,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every day', "campation-postoffice" ) ),
        );
    }
    if ( !isset( $schedules['weekly'] ) ) {
        $schedules['weekly'] = array(
            'interval' => 60 * 60 * 24 * 7,
            'display'  => htmlspecialchars_decode( esc_html__( 'Once every week', "campation-postoffice" ) ),
        );
    }
    return $schedules;
}

// activate cron jobs on plugin activation (hooked in main plugin file)
function cptn_po_cron_activate()
{
    // clear old crons before 1.1.8
    wp_clear_scheduled_hook( 'cptn_po_1minute_cron' );
    wp_clear_scheduled_hook( 'cptn_po_5minute_cron' );
    wp_clear_scheduled_hook( 'cptn_po_15minute_cron' );
    wp_clear_scheduled_hook( 'cptn_po_30minute_cron' );
    wp_clear_scheduled_hook( 'cptn_po_hourly_cron' );
    wp_clear_scheduled_hook( 'cptn_po_daily_cron' );
    wp_clear_scheduled_hook( 'cptn_po_weekly_cron' );
    wp_clear_scheduled_hook( 'cptn_po_monthly_cron' );
    wp_clear_scheduled_hook( 'cptn_po_yearly_cron' );
    // new crons since 1.1.8
    if ( !wp_next_scheduled( 'cptn_po_message_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 60, 'every-1-minutes', 'cptn_po_message_cron' );
    }
    if ( !wp_next_scheduled( 'cptn_po_other_queues_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 2 * 60, 'every-1-minutes', 'cptn_po_other_queues_cron' );
    }
    if ( !wp_next_scheduled( 'cptn_po_recipient_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 2 * 60, 'every-1-minutes', 'cptn_po_recipient_cron' );
    }
    if ( !wp_next_scheduled( 'cptn_po_data_reset_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 2 * 60, 'every-1-minutes', 'cptn_po_data_reset_cron' );
    }
    if ( !wp_next_scheduled( 'cptn_po_statistics_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 15 * 60, 'every-15-minutes', 'cptn_po_statistics_cron' );
    }
    if ( !wp_next_scheduled( 'cptn_po_developer_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 30 * 60, 'daily', 'cptn_po_developer_cron' );
    }
    if ( !wp_next_scheduled( 'cptn_po_purge_cron' ) ) {
        wp_schedule_event( strtotime( date( "Y-m-d H:i:00" ) ) + 24 * 60 * 60, 'daily', 'cptn_po_purge_cron' );
    }
    // ******************************** END CPTNPRO
    error_log( "Campation PostOffice Cron activated" );
    cptn_po_update_option( "cptn_po_cron_running", "1" );
    // start all 1 minute tasks
    cptn_po_message_tasks();
    cptn_po_other_queues_tasks();
    cptn_po_recipient_check_tasks();
}

// deactivate cron on plugin activation (hooked in main plugin file)
function cptn_po_cron_deactivate()
{
    wp_clear_scheduled_hook( 'cptn_po_message_cron' );
    wp_clear_scheduled_hook( 'cptn_po_other_queues_cron' );
    wp_clear_scheduled_hook( 'cptn_po_recipient_cron' );
    wp_clear_scheduled_hook( 'cptn_po_data_reset_cron' );
    wp_clear_scheduled_hook( 'cptn_po_statistics_cron' );
    wp_clear_scheduled_hook( 'cptn_po_developer_cron' );
    wp_clear_scheduled_hook( 'cptn_po_purge_cron' );
    // ******************************** END CPTNPRO
    error_log( "Campation PostOffice Cron deactivated" );
    cptn_po_update_option( "cptn_po_cron_running", "0" );
}

// start cronjob via independant slave
include "includes/cptn-po-cron-slave.php";
function cptn_po_start_cron_slave( $job = "" )
{
    global  $wpdb ;
    $task = array(
        'job' => $job,
    );
    $result = wp_remote_post( esc_url( home_url( "/", "https" ) . 'cptnpocronslave' ), array(
        'method'      => 'POST',
        'sslverify'   => false,
        'timeout'     => 1,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => false,
        'headers'     => array(),
        'body'        => $task,
        'cookies'     => array(),
    ) );
}

// *********************************************
//                    CRON JOBS
// *********************************************
// message send
include "cptn-po-cron-message-send.php";
include "includes/cptn-po-cron-message-slave.php";
add_action( 'cptn_po_message_cron', 'cptn_po_message_tasks' );
function cptn_po_message_tasks()
{
    global  $wpdb ;
    // reset queue mails old embargo
    $query = "UPDATE " . $wpdb->prefix . "cptn_po_queue\r\n\t\t        SET embargo = NULL,\r\n\t\t\t\t    embargo_reason = NULL\r\n\t\t\t  WHERE UNIX_TIMESTAMP(embargo) < UNIX_TIMESTAMP(NOW())\r\n\t\t\t     OR executed IS NOT NULL\r\n\t\t\t\t   ";
    $wpdb->query( $query );
    if ( $wpdb->last_error ) {
        error_log( $wpdb->last_error );
    }
    // reset queue mails without execution that belong to old or not existing slaves
    $query = "UPDATE " . $wpdb->prefix . "cptn_po_queue cpq\r\n\t           LEFT JOIN " . $wpdb->prefix . "cptn_po_slave cps ON cps.id = cpq.slave \r\n\t  \t\t    SET cpq.slave = NULL\r\n\t\t\t  WHERE (cps.created <= DATE_SUB(NOW(), INTERVAL 5 minute)\r\n\t\t\t\t    OR cps.created IS NULL)\r\n\t\t\t    AND cpq.executed IS NULL\r\n\t\t\t\tAND cpq.slave IS NOT NULL\r\n\t\t\t\t   ";
    $wpdb->query( $query );
    if ( $wpdb->last_error ) {
        error_log( $wpdb->last_error );
    }
    // remove old slaves
    $query = "DELETE FROM " . $wpdb->prefix . "cptn_po_slave\r\n\t\t      WHERE created <= DATE_SUB(NOW(), INTERVAL 15 minute)\r\n\t\t\t    ";
    $wpdb->query( $query );
    if ( $wpdb->last_error ) {
        error_log( $wpdb->last_error );
    }
    ob_start();
    if ( intval( cptn_po_get_option( "cptn_po_opt_email_blocked" ) ) != 1 ) {
        cptn_po_start_cron_slave( "message" );
    }
    // runs 55 sec through slave
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_message_send -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}

// check for other mail queues and add to PO queue (e.g. CiviCRM)
include "cptn-po-cron-other-queues.php";
add_action( 'cptn_po_other_queues_cron', 'cptn_po_other_queues_tasks' );
function cptn_po_other_queues_tasks()
{
    global  $wpdb ;
    ob_start();
    if ( intval( cptn_po_get_option( "cptn_po_opt_email_blocked" ) ) != 1 ) {
        cptn_po_start_cron_slave( "otherqueue" );
    }
    // runs 55 sec through slave
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_other_queues -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}

// check recipients
include "cptn-po-cron-recipient-check.php";
add_action( 'cptn_po_recipient_cron', 'cptn_po_recipient_check_tasks' );
function cptn_po_recipient_check_tasks()
{
    global  $wpdb ;
    ob_start();
    cptn_po_start_cron_slave( "recipient" );
    // runs 55 sec through slave
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_recipient_check -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}

// reset periodical data
include "cptn-po-cron-periodical-data-reset.php";
add_action( 'cptn_po_data_reset_cron', 'cptn_po_periodical_data_reset_tasks' );
function cptn_po_periodical_data_reset_tasks()
{
    global  $wpdb ;
    ob_start();
    cptn_po_cron_periodical_data_reset();
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_periodical_data_reset -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}

// update statistical data
include "cptn-po-cron-update-statistics.php";
add_action( 'cptn_po_statistics_cron', 'cptn_po_update_statistics_tasks' );
function cptn_po_update_statistics_tasks()
{
    global  $wpdb ;
    ob_start();
    cptn_po_cron_update_statistics();
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_update_statistics -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}

// purge queue
include "cptn-po-cron-purge-queue.php";
add_action( 'cptn_po_purge_cron', 'cptn_po_purge_queue_tasks' );
function cptn_po_purge_queue_tasks()
{
    global  $wpdb ;
    ob_start();
    if ( intval( get_option( "cptn_po_opt_purge_queue_days" ) ) > 0 ) {
        cptn_po_cron_purge_queue();
    }
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_purge_queue -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}

// send data to developer
include_once "cptn-po-cron-developer-report.php";
add_action( 'cptn_po_developer_cron', 'cptn_po_developer_report_tasks' );
function cptn_po_developer_report_tasks()
{
    global  $wpdb ;
    ob_start();
    if ( intval( get_option( "cptn_po_opt_info_developer" ) ) == 1 ) {
        cptn_po_cron_developer_report();
    }
    $output = ob_get_contents();
    ob_end_clean();
    if ( $output ) {
        error_log( "Session error: cptn_po_cron_developer_report -> " . strlen( $output ) . " bytes (" . $output . ")" );
    }
}
